#!/usr/bin/env python3

import typing
import asyncio
import re
from forge.tasks import wait_cancelable
from forge.acquisition.instrument.streaming import StreamingInstrument, StreamingContext, CommunicationsError
from forge.acquisition.instrument.parse import parse_number

_RECORD = re.compile(rb"PS=\s*(\d+\.\d+)V?.*Bat=\s*(\d+\.\d+)V?[\s,]+(\d+\.\d+)A?.*Sol=\s*(\d+\.\d+)V?")


class Instrument(StreamingInstrument):
    INSTRUMENT_TYPE = "pwrgate"
    MANUFACTURER = "WMR"
    MODEL = "EpicPWRgate"
    DISPLAY_LETTER = "P"
    TAGS = frozenset({"aerosol", "pwrgate"})

    def __init__(self, context: StreamingContext):
        super().__init__(context)

        self.data_Vsupply = self.input("Vsupply")
        self.data_Vbattery = self.input("Vbattery")
        self.data_Vsolar = self.input("Vsolar")
        self.data_Acharger = self.input("Acharger")

        self.instrument_report = self.report(
            self.variable(self.data_Vsupply, "supply_voltage", code="V1", attributes={
                'long_name': "input supply voltage",
                'units': "V",
                'C_format': "%5.2f"
            }),
            self.variable(self.data_Vbattery, "battery_voltage", code="V2", attributes={
                'long_name': "current battery voltage",
                'units': "V",
                'C_format': "%5.2f"
            }),
            self.variable(self.data_Vsolar, "solar_voltage", code="V3", attributes={
                'long_name': "solar panel input voltage",
                'units': "V",
                'C_format': "%5.2f"
            }),

            self.variable(self.data_Acharger, "charger_current", code="A", attributes={
                'long_name': "current being supplied to the battery for charging",
                'units': "A",
                'C_format': "%4.2f"
            }),
        )

    async def start_communications(self) -> None:
        if self.writer:
            # Send returns to try and exit parameter setting mode
            self.writer.write(b"\r\r")
            await self.writer.drain()
            await self.drain_reader(1.0)
            self.writer.write(b"\r")

        # Flush the first record
        await self.drain_reader(0.5)
        await wait_cancelable(self.read_line(), 10.0)

        # Process a valid record
        await self.communicate()

    async def communicate(self) -> None:
        # Allow unmatched lines, but require one within the timeout
        async def get_valid_line() -> re.Match:
            while True:
                line: bytes = await self.read_line()
                match = _RECORD.search(line)
                if match:
                    return match

        match: re.Match = await wait_cancelable(get_valid_line(), 10.0)

        self.data_Vsupply(parse_number(match.group(1)))
        self.data_Vbattery(parse_number(match.group(2)))
        self.data_Acharger(parse_number(match.group(3)))
        self.data_Vsolar(parse_number(match.group(4)))

        self.instrument_report()


def main():
    from forge.acquisition.instrument.streaming import launch
    launch(Instrument)

